﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using PersonLib;
using System.Web.Script.Serialization;

namespace PersonCollectionTest
{
    class Program
    {
        static void Main(string[] args)
        {
            // LIST
            List<Person> pList = new List<Person>();
            PersonFileImporter fileImporter = new PersonFileImporter("Test.csv", ref pList);
            Console.WriteLine("\nTest.csv 파일로딩 후 출력");
            pList.ForEach(Print);

            pList.Add(new Person("둘리", 1, "123-4568-9012", "대한민국"));
            pList.Add(new Person("희동이", 3, "000-0000-0000", "서울"));
            pList.Add(new Person("고길동", 2, "000-0000-0000", "서울"));
            pList.Add(new Person("도우너", 5, "000-0000-0000", "온따빠야별"));
            pList.Add(new Person("또치", 4, "000-0000-0000", "라스베가스"));
            Console.WriteLine("\n둘리 희동이 고길동 도우너 또치 추가 후");
            pList.Insert(2, new Person("마이콜", 6, "000-0000-0000", "서울"));
            Console.WriteLine("\n마이콜을 삽입 후 출력");
            pList.ForEach(Print);

            // JSON serialize
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            var json = serializer.Serialize(pList.ToArray());
            Console.WriteLine(json);

            // JSON deserialize
            JavaScriptSerializer deserializer = new JavaScriptSerializer();
            var loadedList = (List<Person>)deserializer.Deserialize(json, pList.GetType());

            // convert every item to Person
            for (int i = 0; i < loadedList.Count; i++)
            {
                var f = (Person)loadedList[i];
                Console.WriteLine(f);
            }

          
            bool result = false;
            foreach (Person p in pList)
            {
                if (p.ID < 10)
                {
                    result = true;
                    break;
                }
            }
            Console.WriteLine("result=" + result);

            result = true;
            foreach (Person p in pList)
            {
                if (p.Name.Length != 2)
                {
                    result = false;
                    break;
                }
            }
            Console.WriteLine("result=" + result);

            // 1. ID가 10 이하인지를 검사 (Delegate 이용)
            bool result1 = pList.TrueForAll(IsLessThanTen);
            Console.WriteLine("result1=" + result1);

            // 2. 무명 대리자(Anonymous delegate)를 사용하여 이름이 두 글자인지 검사
            bool result2 = pList.TrueForAll(delegate(Person p)
            {
                return p.Name.Length == 2;
            });
            Console.WriteLine("result2=" + result2);

            // 3. LINQ를 사용하여 ID가 10 이하인지를 검사
            bool result3 = pList.TrueForAll(p => (p.ID < 10));
            Console.WriteLine("result3=" + result3);

            // pList에서 search와 일치하는 Person을 찾아서 index를 넘긴다
            Person searchPerson = new Person("또치", 4, "000-0000-0000", "라스베가스");
            int index = pList.FindIndex(delegate(Person o)
            {
                return o == searchPerson;
            });
            Console.WriteLine(index);

            Person found = pList.Find(delegate(Person o)
            {
                return o == searchPerson;
            });
            Console.WriteLine(found.ToString());

            // remove
            pList.Remove(new Person("마이콜", 6, "000-0000-0000", "서울"));
            Console.WriteLine("마이콜을 지우고 출력");
            pList.ForEach(Print);

            // 이름순으로 정렬 - Person 클래스에 IComparable<Person>의 int CompareTo(Person other)
            pList.Sort();
            Console.WriteLine("이름순으로 일반정렬");
            pList.ForEach(Print);

            // 이름순으로 정렬
            pList.Sort(ComparePersonByName);
            Console.WriteLine("이름순으로 일반정렬");
            pList.ForEach(Print);
            pList.Sort(new PersonNameComparer());
            Console.WriteLine("이름순으로 일반정렬");
            pList.ForEach(Print);

            // ID순으로 정렬
            pList.Sort(ComparePersonByID);
            pList.Sort(new PersonIDComparer());
            pList.ForEach(Print);

            // 주소순으로 정렬
            pList.Sort(ComparePersonByAddress);
            pList.Sort(new PersonAddressComparer());
            pList.ForEach(Print);

            // 전화번호순으로 정렬
            pList.Sort(ComparePersonByPhone);
            pList.Sort(new PersonPhoneComparer());
            pList.ForEach(Print);

            // Lambda 로 이름순으로 정렬
            pList.Sort((p, q) => p.CompareTo(q));
            Console.WriteLine();
            Console.WriteLine();
            Console.WriteLine("Lambda 이름순 정렬");
            pList.ForEach(Print);
            // Lambda 로 ID순으로 정렬
            pList.Sort((p, q) => p.ID.CompareTo(q.ID));
            Console.WriteLine("Lambda ID순 정렬");
            pList.ForEach(Print);

            fileImporter.WriteFile("Out.csv");

            // ARRAYLIST
            ArrayList aList = new ArrayList();
            aList.Add(new Person("둘리", 1, "123-4568-9012", "대한민국"));
            aList.Add(new Person("희동이", 3, "000-0000-0000", "서울"));
            aList.Add(new Person("고길동", 2, "000-0000-0000", "서울"));

            Console.WriteLine();
            Console.WriteLine("arraylist 출력");
            for (int i = 0; i < aList.Count; i++)
            {
                Console.WriteLine(aList[i].ToString());
            }

            Console.WriteLine("희동이 삭제 후 출력");
            aList.RemoveAt(1); // 희동이 삭제
            foreach (Person p in aList)
            {
                Console.WriteLine(p.ToString());
            }

            // HASHTABLE
            Hashtable hList = new Hashtable();
            hList.Add("1", new Person("둘리", 1, "123-4568-9012", "대한민국"));
            hList.Add("3", new Person("희동이", 3, "000-0000-0000", "서울"));
            hList.Add("2", new Person("고길동", 2, "000-0000-0000", "서울"));
            hList["4"] = new Person("또치", 4, "000-0000-0000", "라스베가스");

            if (!hList.ContainsKey("5"))
            {
                hList.Add("5", new Person("도우너", 5, "000-0000-0000", "온따빠야별"));
            }

            foreach (DictionaryEntry de in hList)
            {
                Console.WriteLine("Key = {0}, Value = {1}", de.Key, de.Value.ToString());
            }

            Console.WriteLine("둘리 삭제 후 출력");
            hList.Remove("1");
            foreach (DictionaryEntry de in hList)
            {
                Console.WriteLine("Key = {0}, Value = {1}", de.Key, de.Value.ToString());
            }

        }

        // predicate delegate
        private static bool IsLessThanTen(Person p)
        {
            return p.ID < 10;
        }

        // action delegate
        private static void Print(Person p)
        {
            Console.WriteLine(p);
        }

        // comparison delegate
        private static int ComparePersonByName(Person p, Person q)
        {
            return p.Name.CompareTo(q.Name);
        }

        // comparison delegate
        private static int ComparePersonByID(Person p, Person q)
        {
            return p.ID.CompareTo(q.ID);
        }

        // comparison delegate
        private static int ComparePersonByAddress(Person p, Person q)
        {
            return p.Address.CompareTo(q.Address);
        }

        // comparison delegate
        private static int ComparePersonByPhone(Person p, Person q)
        {
            return p.Phone.CompareTo(q.Phone);
        }
    }
}
